//  imagetype.cpp -- template and derived ESC/I ESC/I-2 protocol commands
//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.

#include "mainwindow.h"

Image_Type::Image_Type(QWidget *parent) :
    QComboBox(parent)
{
    changed = false;
}

void Image_Type::initialize(){
    changed = true;
}

void Image_Type::update_ui(INT_SETTING_VALUE image_type_para){

    if(changed == true){
        this->clear();
        if(image_type_para.capability.supportLevel == kSDISupportLevelAvailable){
            this->set_enabled(true);
            this->add_item(image_type_para.capability.list, image_type_para.capability.countOfList);
            this->select_item(image_type_para);
        }else {
            this->set_enabled(false);
        }
        changed = false;
    }
}

void Image_Type::set_enabled(bool enabled){

    if(enabled == true){
        this->setEnabled(true);
    }else {
        this->setEnabled(false);
    }
}

void Image_Type::add_item(int32_t item_list[], int32_t list_size){

    int i = 0;
    while(i < list_size){
        if(item_list[i] == ColorTypeAuto){
            this->insertItem(i, TranslationString::GetString().translation_str.MAINSETTING_017, ColorTypeAuto);
        }else if(item_list[i] == ColorTypeRGB24){ //ColorType
            this->insertItem(i, TranslationString::GetString().translation_str.MAINSETTING_014, ColorTypeRGB24);
        }else if(item_list[i] == ColorTypeMono8){
            this->insertItem(i, TranslationString::GetString().translation_str.MAINSETTING_015, ColorTypeMono8);
        }else if(item_list[i] == ColorTypeMono1){
            this->insertItem(i, TranslationString::GetString().translation_str.MAINSETTING_016, ColorTypeMono1);
        }else if(item_list[i] == ColorTypeRGB48){ // not supported
            this->insertItem(i, "Color (48 bit)", ColorTypeRGB48);
        }else if(item_list[i] == ColorTypeMono16){ // not supported
            this->insertItem(i, "Grayscale (16 bit)", ColorTypeMono16);
        }
        i++;
    }
}

void Image_Type::select_item(INT_SETTING_VALUE image_type_para){

    int list_size = image_type_para.capability.countOfList;

    int i = 0;
    while(i < list_size){
        if(image_type_para.capability.list[i] == image_type_para.select){
            this->setCurrentIndex(i);
        }
        i++;
    }
}

void Image_Type::item_event(int value, INT_SETTING_VALUE* document_source_para)
{
    QVariant select_item = this->itemData(value);

    if(select_item == ColorTypeAuto){
        document_source_para->select = ColorTypeAuto;
    }else if(select_item == ColorTypeRGB24){
        document_source_para->select = ColorTypeRGB24;
    }else if(select_item == ColorTypeMono8){
        document_source_para->select = ColorTypeMono8;
    }else if(select_item == ColorTypeMono1){
        document_source_para->select = ColorTypeMono1;
    }else if(select_item == ColorTypeRGB48){
        document_source_para->select = ColorTypeRGB48;
    }else if(select_item == ColorTypeMono16){
        document_source_para->select = ColorTypeMono16;
    }
    changed = true;
}
