//  lastusedsettings.h -- template and derived ESC/I ESC/I-2 protocol commands
//  Copyright (C) 2018-2019  SEIKO EPSON CORPORATION
//
//  License: GPL-3.0+
//  Author : SEIKO EPSON CORPORATION
//
//  This file is part of the 'Epson Scan 2' package.
//  This package is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License or, at
//  your option, any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//  You ought to have received a copy of the GNU General Public License
//  along with this package.  If not, see <http://www.gnu.org/licenses/>.

#ifndef LASTUSEDSETTINGS_H
#define LASTUSEDSETTINGS_H

#include "ESJsonUtils.h"
#include "supervisor.h"
#include "configuration.h"

class LastUsedSettings
{
private:
    std::string PreferredInfoFilePath;
    std::string SettingsFilePath;
public:
    LastUsedSettings();

    //前回接続デバイス関連処理
    bool CheckLastUsedDevice();
    bool ReadPreferredInfo();
    bool SelectUSBDevice(std::string display_name);
    bool SelectNetworkDevice(std::string ip);
    void CreatePreferredInfo(SDIDeviceInfo dev_info);
    void DeletePreferredInfo(void);

    //CommonSettingsファイル関連
    bool CheckCommonSettingsFile();
    CONFIGURATION ReadCommonSettingsFile();
    void CreateCommonSettingsFile(Configuration& config);
    void DeleteCommonSettingsFile();
    CONFIGURATION ReadMainWindowInfo();

    //前回設定の保存関連
    bool CreateSettingsFile(std::string model_id, const SCANPARA device_data);
    void CreateSettingsDict(std::string user_settngs_file, const SCANPARA device_data);
    bool ReadSettingsFile(std::string model_id, SCANPARA* device_data);

    bool ReadCmdSettingsFile(std::string settings_file_path, SCANPARA* device_data);
    void SetDeviceData(std::string settings_file_path, SCANPARA* device_data);
};

#endif // LASTUSEDSETTINGS_H
